@@ -51,7 +51,7 @@
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 #include "rotator.h"
 #include "gltrackball.h"
 #include "extrusion.h"
@@ -224,8 +224,8 @@ static void Create_Texture(ModeInfo *mi, const char *filename)
     }
   else
     {
-      XImage *ximage = xpm_file_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                           MI_COLORMAP (mi), filename);
+      XImage *ximage = file_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                       filename);
       if (!ximage)
         goto BUILTIN;
       image  = (GLubyte *) ximage->data;
