@@ -119,15 +119,10 @@ ModStruct   planet_description =
  "Animates texture mapped sphere (planet)", 0, NULL};
 #endif
 
-# ifdef __GNUC__
-  __extension__  /* don't warn about "string length is greater than the length
-                    ISO C89 compilers are required to support" when including
-                    the following XPM file... */
-# endif
-#include "../images/earth.xpm"
-#include "../images/earth_night.xpm"
+#include "images/gen/earth_png.h"
+#include "images/gen/earth_night_png.h"
 
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 #include "rotator.h"
 #include "gltrackball.h"
 
@@ -165,10 +160,10 @@ static planetstruct *planets = NULL;
 
 /* Set up and enable texturing on our object */
 static void
-setup_xpm_texture (ModeInfo *mi, char **xpm_data)
+setup_xpm_texture (ModeInfo *mi, const unsigned char *data, unsigned long size)
 {
-  XImage *image = xpm_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                  MI_COLORMAP (mi), xpm_data);
+  XImage *image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                        data, size);
   char buf[1024];
   clear_gl_error();
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
@@ -177,10 +172,7 @@ setup_xpm_texture (ModeInfo *mi, char **xpm_data)
   */
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                image->width, image->height, 0,
-               GL_RGBA,
-               /* GL_UNSIGNED_BYTE, */
-               GL_UNSIGNED_INT_8_8_8_8_REV,
-               image->data);
+               GL_RGBA, GL_UNSIGNED_BYTE, image->data);
   sprintf (buf, "builtin texture (%dx%d)", image->width, image->height);
   check_gl_error(buf);
 }
@@ -193,8 +185,7 @@ setup_file_texture (ModeInfo *mi, char *filename)
   Visual *visual = mi->xgwa.visual;
   char buf[1024];
 
-  Colormap cmap = mi->xgwa.colormap;
-  XImage *image = xpm_file_to_ximage (dpy, visual, cmap, filename);
+  XImage *image = file_to_ximage (dpy, visual, filename);
   if (!image) return False;
 
   clear_gl_error();
@@ -202,10 +193,7 @@ setup_file_texture (ModeInfo *mi, char *filename)
   glPixelStorei(GL_UNPACK_ROW_LENGTH, image->width);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                image->width, image->height, 0,
-               GL_RGBA,
-               /* GL_UNSIGNED_BYTE, */
-               GL_UNSIGNED_INT_8_8_8_8_REV,
-               image->data);
+               GL_RGBA, GL_UNSIGNED_BYTE, image->data);
   sprintf (buf, "texture: %.100s (%dx%d)",
            filename, image->width, image->height);
   check_gl_error(buf);
@@ -233,7 +221,7 @@ setup_texture(ModeInfo * mi)
 	  !strcmp(which_image, "BUILTIN"))
     {
     BUILTIN1:
-      setup_xpm_texture (mi, earth_xpm);
+      setup_xpm_texture (mi, earth_png, sizeof(earth_png));
     }
   else
     {
@@ -258,7 +246,7 @@ setup_texture(ModeInfo * mi)
 	  !strcmp(which_image2, "BUILTIN"))
     {
     BUILTIN2:
-      setup_xpm_texture (mi, earth_night_xpm);
+      setup_xpm_texture (mi, earth_night_png, sizeof(earth_night_png));
     }
   else
     {
@@ -287,12 +275,18 @@ init_stars (ModeInfo *mi)
   int max_size = 3;
   GLfloat inc = 0.5;
   int steps = max_size / inc;
+  GLfloat scale = 1;
+
+  if (MI_WIDTH(mi) > 2560) {  /* Retina displays */
+    scale *= 2;
+    nstars /= 2;
+  }
 
   gp->starlist = glGenLists(1);
   glNewList(gp->starlist, GL_COMPILE);
   for (j = 1; j <= steps; j++)
     {
-      glPointSize(inc * j);
+      glPointSize(inc * j * scale);
       glBegin (GL_POINTS);
       for (i = 0; i < nstars / steps; i++)
         {
@@ -333,7 +327,7 @@ reshape_planet (ModeInfo *mi, int width, int height)
   glViewport(0, 0, (GLint) width, (GLint) height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  glFrustum(-1.0, 1.0, -h, h, 5.0, 100.0);
+  glFrustum(-1.0, 1.0, -h, h, 5.0, 200.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef(0.0, 0.0, -40);
