@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -70,6 +70,10 @@
 
 #ifdef HAVE_JWXYZ
 # include "jwxyz.h"
+# include <string.h> /* X11/Xos.h brings this in. */
+/* From utils/visual.c. */
+# define DEFAULT_VISUAL	-1
+# define GL_VISUAL	-6
 #else  /* real X11 */
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
@@ -83,6 +87,7 @@
 
 #ifdef HAVE_ANDROID
  /* So that hacks' debug output shows up in logcat... */
+# undef  fprintf
 # define fprintf(S, ...) Log(__VA_ARGS__)
 #endif
 
@@ -108,6 +113,7 @@
 #include "grabscreen.h"
 #include "visual.h"
 #include "fps.h"
+#include "font-retry.h"
 
 /* Be Posixly correct */
 #undef  bzero
@@ -138,8 +144,12 @@ struct xscreensaver_function_table {
   void           (*free_cb)    (Display *, Window, void *);
   void           (*fps_cb)     (Display *, Window, fps_state *, void *);
 
+# ifndef HAVE_JWXYZ
   Visual *       (*pick_visual_hook) (Screen *);
   Bool           (*validate_visual_hook) (Screen *, const char *, Visual *);
+# else
+  int            visual;
+# endif
 
 };
 
